package repository

import (
	"gitlab.com/gitlab-org/gitaly/v14/client"
	"gitlab.com/gitlab-org/gitaly/v14/internal/git"
	"gitlab.com/gitlab-org/gitaly/v14/internal/git/catfile"
	"gitlab.com/gitlab-org/gitaly/v14/internal/git/localrepo"
	"gitlab.com/gitlab-org/gitaly/v14/internal/git/repository"
	"gitlab.com/gitlab-org/gitaly/v14/internal/git2go"
	"gitlab.com/gitlab-org/gitaly/v14/internal/gitaly/config"
	"gitlab.com/gitlab-org/gitaly/v14/internal/gitaly/rubyserver"
	"gitlab.com/gitlab-org/gitaly/v14/internal/gitaly/storage"
	"gitlab.com/gitlab-org/gitaly/v14/internal/gitaly/transaction"
	"gitlab.com/gitlab-org/gitaly/v14/proto/go/gitalypb"
)

type server struct {
	gitalypb.UnimplementedRepositoryServiceServer
	ruby          *rubyserver.Server
	conns         *client.Pool
	locator       storage.Locator
	txManager     transaction.Manager
	gitCmdFactory git.CommandFactory
	cfg           config.Cfg
	binDir        string
	loggingCfg    config.Logging
	catfileCache  catfile.Cache
	git2go        git2go.Executor
}

// NewServer creates a new instance of a gRPC repo server
func NewServer(
	cfg config.Cfg,
	rs *rubyserver.Server,
	locator storage.Locator,
	txManager transaction.Manager,
	gitCmdFactory git.CommandFactory,
	catfileCache catfile.Cache,
	connsPool *client.Pool,
) gitalypb.RepositoryServiceServer {
	return &server{
		ruby:          rs,
		locator:       locator,
		txManager:     txManager,
		gitCmdFactory: gitCmdFactory,
		conns:         connsPool,
		cfg:           cfg,
		binDir:        cfg.BinDir,
		loggingCfg:    cfg.Logging,
		catfileCache:  catfileCache,
		git2go:        git2go.NewExecutor(cfg, locator),
	}
}

func (s *server) localrepo(repo repository.GitRepo) *localrepo.Repo {
	return localrepo.New(s.gitCmdFactory, s.catfileCache, repo, s.cfg)
}
